# load of R-OpenFLUID package
library("ROpenFLUID")    # should be done only once 

# definition of the paths and OpenFLUID simulations
OF.dirBase = "/home/openfluid/formation/datasets/tuto-ropenfluid/"   # directory of OpenFLUID project
OF.dirIn   = paste(OF.dirBase,"IN",sep="")
OF.dirOut  = paste(OF.dirBase,"OUT",sep="")
OF.simu    = OpenFLUID.openDataset(OF.dirIn)  # OpenFLUID simulation
OpenFLUID.setCurrentOutputDir(OF.dirOut)      # directory of output files
OF.deltaT  = OpenFLUID.getDeltaT(OF.simu)     # time step duration

# test of OpenFLUID simulation
OpenFLUID.addVariablesExportAsCSV(OF.simu,'RS')
OpenFLUID.addVariablesExportAsCSV(OF.simu,'SU')
OpenFLUID.runSimulation(OF.simu)

# loading of some results
OF.H.rain          = OpenFLUID.loadResult(OF.simu,"SU",5,"water.atm-surf.H.rain")
OF.H.runoff        = OpenFLUID.loadResult(OF.simu,"SU",5,"water.surf.H.runoff")
OF.H.infiltration  = OpenFLUID.loadResult(OF.simu,"SU",5,"water.surf.H.infiltration")
OF.Q.downstream    = OpenFLUID.loadResult(OF.simu,"RS",3,"water.surf.Q.downstream-rs")
OF.H.level         = OpenFLUID.loadResult(OF.simu,"RS",3,"water.surf.H.level-rs")

# screen output of some results
print(paste("cumulative downstream : ",as.character(sum(OF.Q.downstream[,2])*OF.deltaT)," m3",sep=""))
print(paste("maximum water level   : ",as.character(max(OF.H.level[,2]))," m",sep=""))

# plot of some results
pdf(paste(OF.dirBase,"plot-R-MHYDAS-SU5.pdf",sep=""))
plot(OF.H.rain[,1],OF.H.rain[,2],type='l',lwd=3,xlab='Date',ylab='Flow [m3.s-1]')
lines(OF.H.infiltration[,1],OF.H.infiltration[,2],type='l',col='red',lwd=2)
lines(OF.H.runoff[,1],OF.H.runoff[,2],type='l',col='blue',lwd=2)
legend("topright",c('Rain on SU5','Runoff on SU5','Infiltration on SU5'),lty=c(1,1,1),lwd=c(3,2,2),col=c('black','red','blue'))
dev.off()

# definition of the function running OpenFLUID with the factors set X
OF.multiRun <- function(X,varOut) {

  out = rep(NA,nrow(X))
  
  # loop on the lines of the factors vector
  for (i in seq(1:nrow(X))) {

    # redefine simulator parameters and input data according to vector X
    OpenFLUID.setSimulatorParam(OF.simu,"water.surf.transfer-rs.hayami","meancel",X[i,1])
    OpenFLUID.setInputData(OF.simu,"SU",5,"thetaini",X[i,2])
    OpenFLUID.setInputData(OF.simu,"RS",3,"width",X[i,3])
    
    # run of OpenFLUID with new simulator parameters and input data
    OpenFLUID.addVariablesExportAsCSV(OF.simu,'RS')
    OpenFLUID.addVariablesExportAsCSV(OF.simu,'SU')
    OpenFLUID.runSimulation(OF.simu)

    # choice of output variables
    if (varOut=="Q") {
      outOF  = OpenFLUID.loadResult(OF.simu,"RS",3,"water.surf.Q.downstream-rs")
      out[i] = sum(outOF[,2])*OpenFLUID.getDeltaT(OF.simu)
      if (i %% 10 == 0) print(paste("cumulative downstream for simulation no ",as.character(i)," : ",as.character(out[i])," m3",sep=""))
    }
    else if (varOut=="H") {
      outOF  = OpenFLUID.loadResult(OF.simu,"RS",3,"water.surf.H.level-rs")
      out[i] = max(outOF[,2])
      if (i %% 10 == 0) print(paste("max water level for simulation no ",as.character(i)," : ",as.character(out[i])," m",sep=""))
    }
  }
  return(out)
}

# test of the multi-simulation function
# definition of the factors set
N = 500
OF.caractFactor = list(
  name = c("Mean celerity on RS","Initial moisture on SU5","Width of RS3"),
  ref  = c(0.6,0.3 ,0.50),
  inf  = c(0.2,0.25,0.35),
  sup  = c(1.0,0.35,0.65)
)
v1 = apply(matrix(runif(3*N),nrow=N,ncol=3),
          MARGIN=1, function(x) {OF.caractFactor$inf + x*(OF.caractFactor$sup-OF.caractFactor$inf)})
v1 = t(v1)
colnames(v1) <- OF.caractFactor$name

# launch of the multi-simulation
#flowOutlet1 = OF.multiRun(v1,"Q")
waterLevel1 = OF.multiRun(v1,"H")

OF.caractFactor = list(
  name = c("Mean celerity on RS","Initial moisture on SU5","Width of RS3"),
  ref  = c(0.6,0.3 ,0.50),
  inf  = c(0.5,0.25,0.35),
  sup  = c(0.7 ,0.35,0.65)
)
v2 = apply(matrix(runif(3*N),nrow=N,ncol=3),
          MARGIN=1, function(x) {OF.caractFactor$inf + x*(OF.caractFactor$sup-OF.caractFactor$inf)})
v2 = t(v2)
colnames(v2) <- OF.caractFactor$name

# launch of the multi-simulation
#flowOutlet2  = OF.multiRun(v2,"Q")
waterLevel2 = OF.multiRun(v2,"H")

OF.caractFactor = list(
  name = c("Mean celerity on RS","Initial moisture on SU5","Width of RS3"),
  ref  = c(0.6,0.3 ,0.50),
  inf  = c(0.2,0.25,0.45),
  sup  = c(1.0,0.35,0.55)
)
v3 = apply(matrix(runif(3*N),nrow=N,ncol=3),
          MARGIN=1, function(x) {OF.caractFactor$inf + x*(OF.caractFactor$sup-OF.caractFactor$inf)})
v3 = t(v3)
colnames(v3) <- OF.caractFactor$name

# launch of the multi-simulation
#flowOutlet3  = OF.multiRun(v3,"Q")
waterLevel3 = OF.multiRun(v3,"H")

# statistical uncertainty analysis
quantile(waterLevel1, probs = c(0.05,0.5,0.95))
quantile(waterLevel2, probs = c(0.05,0.5,0.95))
quantile(waterLevel3, probs = c(0.05,0.5,0.95))

pdf(paste(OF.dirBase,"hist-R-MHYDAS-RS3-1.pdf",sep=""))
for (i in seq(1,ncol(v1))) {
  hist(v1[,i])
  title(sub=OF.caractFactor$name[i])
}
hist(flowOutlet1)
hist(waterLevel1,xlim=c(0.0075,0.015))
dev.off()

pdf(paste(OF.dirBase,"hist-R-MHYDAS-RS3-2.pdf",sep=""))
for (i in seq(1,ncol(v2))) {
  hist(v2[,i])
  title(sub=OF.caractFactor$name[i])
}
hist(flowOutlet2)
hist(waterLevel2,xlim=c(0.0075,0.015))
dev.off()

pdf(paste(OF.dirBase,"hist-R-MHYDAS-RS3-3.pdf",sep=""))
for (i in seq(1,ncol(v3))) {
  hist(v3[,i])
  title(sub=OF.caractFactor$name[i])
}
hist(flowOutlet3)
hist(waterLevel3,xlim=c(0.0075,0.015))
dev.off()

# load of sensitivity package
library("sensitivity")   # should be done only once

# Morris sensitivity analysis
OF.morris.Q <- morris(model=OF.multiRun,
                             factors=as.character(OF.caractFactor$name),r=N/2,
                             design=list(type="oat", levels=25, grid.jump=12),
                             binf=c(0.1,0.05,0.1),
                             bsup=c(1.0,0.35,1.0),
                             varOut="Q")
# Morris sensitivity analysis
OF.morris.H <- morris(model=OF.multiRun,
                    factors=as.character(OF.caractFactor$name),r=N/2,
                    design=list(type="oat", levels=25, grid.jump=12),
                    binf=c(0.1,0.05,0.1),
                    bsup=c(1.0,0.35,1.0),
                    varOut="H")

pdf(paste(OF.dirBase,"morris.pdf",sep=""))
plot(OF.morris.Q,xlim=c(0,1),ylim=c(0,1))
title(main="Morris sensitivity analysis on the downstream")
plot(OF.morris.H,xlim=c(0,1),ylim=c(0,1))
title(main="Morris sensitivity analysis on the water level")
dev.off()

# FAST sensitivity analysis
OF.bounds <- apply(cbind((c(0.1,0.05,0.1)), (c(1.0,0.35,1.0))),MARGIN=1,function(x){list(min=x[1],max=x[2])} )

OF.fast99.Q <- fast99(model=OF.multiRun,
                      factors=as.character(OF.caractFactor$name),
                      n=N/2,
                      q="qunif",
                      q.arg=OF.bounds,
                      varOut="Q")
OF.fast99.H <- fast99(model=OF.multiRun,
                      factors=as.character(OF.caractFactor$name),
                      n=N/2,
                      q="qunif",
                      q.arg=OF.bounds,
                      varOut="H")

pdf(paste(OF.dirBase,"fast99.pdf",sep=""))
plot(OF.fast99.Q)
title(main="FAST sensitivity analysis on the downstream")
plot(OF.fast99.H)
title(main="FAST sensitivity analysis on the water level")
dev.off()

# effect of variation
OF.factorDemoAS = rbind(OF.caractFactor$ref,OF.caractFactor$ref*c(1,0.75,1),OF.caractFactor$ref*c(1,1,0.75))
OF.demoASQ = OF.multiRun(OF.factorDemoAS,'Q')
OF.demoASH = OF.multiRun(OF.factorDemoAS,'H')
print(OF.demoASQ / OF.demoASQ[1])
print(OF.demoASH / OF.demoASH[1])

# model optimization
# definition of the function running OpenFLUID with width of RS3 mean celerity as unknown variables
OF.singleRun <- function(X,ref) {

    # redefine simulator parameters and input data according to vector X
    OpenFLUID.setSimulatorParam(OF.simu,"water.surf.transfer-rs.hayami","meancel",X[1])
    OpenFLUID.setInputData(OF.simu,"RS",3,"width",X[2])
    
    # run of OpenFLUID with new simulator parameters and input data
    OpenFLUID.addVariablesExportAsCSV(OF.simu,'RS')
    OpenFLUID.runSimulation(OF.simu)

    # choice of output variables
    outOF = OpenFLUID.loadResult(OF.simu,"RS",3,"water.surf.H.level-rs")
    out   = sqrt(sum((outOF[,2]-ref[,2])**2))
    print(paste("simulation :",X[1],X[2],"RMSE",out,sep=" "))
    return(out)
}

# reference output definition
OF.refOptim=OpenFLUID.loadResultFile(paste(OF.dirBase,"measured-level.csv",sep=""))

pdf(paste(OF.dirBase,"R-MHYDAS-measured-level.pdf",sep=""))
plot(OF.refOptim,xlab='Date',ylab='Water level [m]')
dev.off()

# run optimization process
OF.parInit = c(0.01,0.2)
OF.factorsOptim=optim(par=OF.parInit,fn=OF.singleRun,method='L-BFGS-B',lower=0.1*OF.parInit,upper=5.0*OF.parInit,ref=OF.refOptim)

pdf(paste(OF.dirBase,"R-MHYDAS-optimized-level.pdf",sep=""))
OpenFLUID.setSimulatorParam(OF.simu,"water.surf.transfer-rs.hayami","meancel",OF.parInit[1])
OpenFLUID.setInputData(OF.simu,"RS",3,"width",OF.parInit[2])
OpenFLUID.addVariablesExportAsCSV(OF.simu,'RS')
OpenFLUID.runSimulation(OF.simu)
plot(OpenFLUID.loadResult(OF.simu,"RS",3,"water.surf.H.level-rs"),type='l',lwd=2,xlab='Date',ylab='Water level [m]')
OpenFLUID.setSimulatorParam(OF.simu,"water.surf.transfer-rs.hayami","meancel",OF.factorsOptim$par[1])
OpenFLUID.setInputData(OF.simu,"RS",3,"width",OF.factorsOptim$par[2])
OpenFLUID.addVariablesExportAsCSV(OF.simu,'RS')
OpenFLUID.runSimulation(OF.simu)
lines(OpenFLUID.loadResult(OF.simu,"RS",3,"water.surf.H.level-rs"),col='red',lwd=2)
points(OF.refOptim,col='blue')
legend("topright",c('Initial parameters','Optimized parameters','Measured'),lty=c(1,1,3),lwd=c(2,2,2),col=c('black','red','blue'))
print(OF.factorsOptim$par)
dev.off()
