/**
  \file ReservoirCSim.cpp
 */


/*
<sim2doc>

</sim2doc>
 */


#include <openfluid/ware/PluggableSimulator.hpp>

extern "C"
{
#include "ReservoirCCalc.h"
}

// =====================================================================
// =====================================================================


DECLARE_SIMULATOR_PLUGIN


// =====================================================================
// =====================================================================


BEGIN_SIMULATOR_SIGNATURE("training.encapsulation.reservoir-c")

DECLARE_NAME("water storage modelling - c language");
DECLARE_DESCRIPTION("this simulator distributes a water height in the different layers of the unsaturated zone");

DECLARE_VERSION("13.05");
DECLARE_STATUS(openfluid::ware::EXPERIMENTAL);

DECLARE_DOMAIN("unsaturated zone");
DECLARE_PROCESS("soil water infiltration");
DECLARE_METHOD("capacitive modeling");
DECLARE_AUTHOR("David Crevoisier","david.crevoisier@supagro.inra.fr");
DECLARE_AUTHOR("Cécile Dagès","cecile.dages@supagro.inra.fr");

// Input data
DECLARE_REQUIRED_INPUTDATA("th_cc","SU","Field capacity","m3/m3")
DECLARE_REQUIRED_INPUTDATA("thetai","SU","Initial water content","m3/m3")
DECLARE_REQUIRED_INPUTDATA("dz","SU","Soil layer depth","m")

// Variables
DECLARE_PRODUCED_VAR("water.surf-uz.theta[]","SU","Soil moisture profile","m3/m3")
DECLARE_PRODUCED_VAR("water.surf-uz.drainage","SU","Drainage below soil profile","m")
DECLARE_PRODUCED_VAR("water.surf-uz.storage","SU","Water storage in the whole soil column","m")
DECLARE_REQUIRED_VAR("water.atm-surf.H.rain","SU","Produced variable, distributed on SU - rainfall height on each SU by time step","m")


// Scheduling
DECLARE_SCHEDULING_DEFAULT;



END_SIMULATOR_SIGNATURE


// =====================================================================
// =====================================================================


/**

 */
class ReservoirCSim : public openfluid::ware::PluggableSimulator
{
  private:

  int m_nCouche;
  // IDVectorValueMap : vecteurs spatialises
  openfluid::core::IDDoubleValueMap m_th_cc;
  openfluid::core::IDVectorValueMap m_thCI;
  openfluid::core::IDVectorValueMap m_dz;


  public:


  ReservoirCSim(): PluggableSimulator()
  {


  }


  // =====================================================================
  // =====================================================================


  ~ReservoirCSim()
  {


  }


  // =====================================================================
  // =====================================================================


  void initParams(const openfluid::ware::WareParams_t& /*Params*/)
  {


  }

  // =====================================================================
  // =====================================================================


  void prepareData()
  {


  }


  // =====================================================================
  // =====================================================================


  void checkConsistency()
  {


  }


  // =====================================================================
  // =====================================================================


  openfluid::base::SchedulingRequest initializeRun()
  {

    openfluid::core::UnitID_t ID;
    openfluid::core::Unit* SU;
    openfluid::core::StringValue str_thetai;
    openfluid::core::StringValue str_dz;
    openfluid::core::DoubleValue th_cc;

    bool lErr;

    OPENFLUID_UNITS_ORDERED_LOOP("SU",SU)
    {

      ID = SU->getID();

      OPENFLUID_GetInputData(SU,"th_cc",th_cc);
      OPENFLUID_GetInputData(SU,"thetai",str_thetai);
      OPENFLUID_GetInputData(SU,"dz",str_dz);

      openfluid::core::VectorValue thetaiVect;
      openfluid::core::VectorValue dzVect;
      str_thetai.toVectorValue(";",thetaiVect);
      str_dz.toVectorValue(";",dzVect);
      m_nCouche = thetaiVect.getSize();

      // pour fortran ThetaiVect.getData()
      // pour Fortran ThetaiVect.getSize()

      // creation des vecteurs déclarés comme attributs
      m_th_cc[ID] = th_cc;
      m_dz[ID]    = openfluid::core::VectorValue(m_nCouche);
      m_thCI[ID]  = openfluid::core::VectorValue(m_nCouche);

      for (int i=0;i<m_nCouche;i++)
        m_thCI[ID][i] = thetaiVect[i];

      for (int i=0;i<m_nCouche;i++)
        m_dz[ID][i] = dzVect[i];

      OPENFLUID_InitializeVariable(SU,"water.surf-uz.theta",m_thCI[ID]);
      OPENFLUID_InitializeVariable(SU,"water.surf-uz.drainage",0.0);
      OPENFLUID_InitializeVariable(SU,"water.surf-uz.storage",0.0);

    }
    return DefaultDeltaT();
  }

  // =====================================================================
  // =====================================================================


  openfluid::base::SchedulingRequest runStep()
  {

    openfluid::core::UnitID_t ID;
    int k;
    double apportActu,drainage,storage;
    openfluid::core::VectorValue Theta;
    openfluid::core::Unit *SU;

    // definition des pas de temps en courant et precedent
    openfluid::core::TimeIndex_t PreviousTimeIndex = OPENFLUID_GetPreviousRunTimeIndex();

    // debut de la boucle d espace
    OPENFLUID_UNITS_ORDERED_LOOP("SU",SU)
    {

      ID = SU->getID();

      OPENFLUID_GetVariable(SU,("water.atm-surf.H.rain"),apportActu);

      OPENFLUID_GetVariable(SU,("water.surf-uz.drainage"),PreviousTimeIndex,drainage);

      // calculs sur un pas de temps

      double *tmp_dz   = m_dz[ID].getData();
      double *tmp_thCI = m_thCI[ID].getData();
      double tmp_th_cc = m_th_cc[ID].get();

      ReservoirCCalc(m_nCouche,tmp_th_cc,tmp_thCI,tmp_dz,&apportActu,&storage,&drainage);

      Theta = openfluid::core::VectorValue(m_nCouche);
      for (k=0;k<m_nCouche;k++)
      {
        Theta[k] = tmp_thCI[k];
      }

      OPENFLUID_AppendVariable(SU,"water.surf-uz.theta",Theta);
      OPENFLUID_AppendVariable(SU,"water.surf-uz.storage",storage);
      OPENFLUID_AppendVariable(SU,"water.surf-uz.drainage",drainage);

    }

    return DefaultDeltaT();
  }

  // =====================================================================
  // =====================================================================


  void finalizeRun()
  {


  }

};


// =====================================================================
// =====================================================================

DEFINE_SIMULATOR_CLASS(ReservoirCSim);
